SUBROUTINE COMMANDLINE()

!   ****************************    NOTICE   **************************** 
 
!    This "Image Quality of Fingerprint" (IQF) software was developed by 
!                            The MITRE Corporation
!    and was produced for the U.S. Government under Contract numbers 
!    J-FBI-12-128, W15P7T-05-C-F600 and W15P7T-07-C-F600 and is subject 
!    to the Rights in Data-General clause FAR 52.227-l4, Alt. IV (DEC 2007).
	
!   Redistribution and use in source and binary forms, with or without
!   modification, are permitted provided that the following conditions are met:
	
!    -Redistribution of source code must retain the above copyright notice,
!    this list of conditions, and the following disclaimer.
	
!    -Redistribution in binary form must reproduce the above copyright
!    notice, this list of conditions and the following disclaimer in the
!    documentation and/or other materials provided with the distribution.
	
!    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
!    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
!    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
!    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
!    OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
!    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
!    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
!    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
!    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
!    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
!    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
      
!   ********************************************************************* 
 
 
! This Subroutine:
! makes calls to the Unix command line retrieval subroutine "getarg", therefore,
! whatever Fortran compiler is used, it must be able to access getarg from a Unix library

      USE GlobalData
      
       
      CHARACTER (LEN=100) ::  ARGV, parm      
      
      ARGC = IARGC()		! arg count
      imgtype = 'N'
          
  ! in following, runtype is keyed to values specified in iqf.f95
  
	IF(ARGC == 0) THEN 
		RETURN
      ELSEIF(ARGC == 1) THEN	! display help info
      		CALL GETARG(1, ARGV)
      		parm = TRIM(ARGV)
      		IF(parm(1:1) == '-') THEN 	! delete any leading dash in argument
          	  	m = LEN(parm)
          	  	parm = parm(2:m)
          	ENDIF
      		IF(parm(1:1)=='h' .OR. parm(1:1)=='H') THEN	
       			IF(parm(2:2)/='h' .AND. parm(2:2)/='H') THEN
       	PRINT*, ' '
       	PRINT*, 'Valid Argument Sets:'
       	PRINT*, 's  ppi  imgfilename   	                   (Single image in PGM format)'   
      	PRINT*, 's  ppi  hdr  width  height  imgfilename   (Single image not in PGM format)' 
      	PRINT*, 'c  InputDataFile                          (Concise output, multiple images)'
       	PRINT*, 'd  InputDataFile                          (Diagnostic output, multiple images)'
      	PRINT*, 'f  InputDataFile                          (Diagnostic output, tenprint cardscans)'
      	PRINT*, 'hh                                         (more Help)' 
      	PRINT*, ' '
        PRINT*, 'above c, d, or f for inked/livescans; for latents use ct or dt' 
      	PRINT*, ' '
      	PRINT*, 'example Windows run (single livescan):  iqf.exe  s   500   fprint.pgm' 
    	PRINT*, 'example Mac run (latents in file):     ./iqf  ct  inputdatafile'
    	PRINT*, ' '
    	PAUSE
    	STOP
   			ELSE
   				CALL HELP()
   				PAUSE
   				STOP
   			ENDIF
   		
   		ELSE
      			PAUSE 'Quitting:   Invalid Argument'
      			STOP 
      		ENDIF
      
     ELSEIF(ARGC == 2) THEN	! multiple image run with InputDataFile
           DO J = 2, ARGC+1	
          	  CALL GETARG(J-1, ARGV)
          	  parm = TRIM(ARGV)
          	  IF(parm(1:1) == '-') THEN 	! delete any leading dash in argument
          	  	m = LEN(parm)
          	  	parm = parm(2:m)
          	  ENDIF
          	  
          	  IF(J == 2) THEN	! 1st arg: s, c, d or j
      		  	IF(parm(1:1) == 'c' .OR. parm(1:1) == 'C' ) THEN
     				runtype = 'c'
     		  	ELSEIF(parm(1:1) == 'd' .OR. parm(1:1) == 'D') THEN
     				runtype = 'd'
     		  	ELSEIF(parm(1:1) == 'f' .OR. parm(1:1) == 'F') THEN
     				runtype = 'f'
			ELSE
    				PRINT*, 'Quitting:  Invalid Argument:  ',parm(1:1)
    				PAUSE
    				STOP 
     		  	ENDIF
     		  	IF(parm(2:2) == 't' .OR. parm(2:2) == 'T') imgtype = 'L'
          	  ELSEIF(J == 3) THEN	 ! 2nd arg = InputDataFileName
			auxdataname = TRIM(parm)
         	  ENDIF
           END DO
  
   ELSEIF(ARGC == 3) THEN   ! single pgm image run 
           DO J = 2, ARGC+1	
          	  CALL GETARG(J-1, ARGV)
          	  parm = TRIM(ARGV)
          	  IF(parm(1:1) == '-') THEN 	! delete any leading dash in argument
          	  	m = LEN(parm)
          	  	parm = parm(2:m)
          	  ENDIF
          	  
          	  IF(J == 2) THEN	! 1st arg = s  for single image run
      		  	IF(parm(1:1) == 's' .OR. parm(1:1) == 'S' ) THEN
     				runtype = 's'
     				IF(parm(2:2)=='l'.OR.parm(2:2)=='L') THEN 
     					PAUSE 'Quitting:  need inputfile for latent image run (ct or dt)'
     					STOP
     				ENDIF 
       		  	ELSE
   				PRINT*, 'Quitting:  Invalid Argument:  ',parm(1:1)
    				PAUSE
    				STOP 
     		  	ENDIF
			IF(parm(2:2) == 't' .OR. parm(2:2) == 'T') imgtype = 'L'
          	 
          	  ELSEIF(J == 3) THEN	 ! 2nd arg = ppi
			READ(parm, fmt='(I6)') ppi
			
          	  ELSEIF(J == 4) THEN 	 ! 3rd arg = imagefilename
          	  	imgID = TRIM(parm)
         	  ENDIF
            END DO
 
   ELSEIF(ARGC == 6) THEN	! single image run but Not pgm format 
           DO J = 2, ARGC+1	
          	  CALL GETARG(J-1, ARGV)
          	  parm = TRIM(ARGV)
          	  IF(parm(1:1) == '-') THEN 	! delete any leading dash in argument
          	  	m = LEN(parm)
          	  	parm = parm(2:m)
          	  ENDIF
         	  IF(J == 2) THEN	! 1st arg = s
      		  	IF(parm(1:1) == 's' .OR. parm(1:1) == 'S' ) THEN
     				runtype = 's' 
     				IF(parm(2:2)=='t'.OR.parm(2:2)=='T') THEN
     					PAUSE 'Quitting:  need inputfile for latent image run (ct or dt)'
     					STOP
     				ENDIF
       		  	ELSE
    				PRINT*, 'Quitting:  Invalid Argument:  ',parm(1:1)
    				PAUSE
    				STOP 
     		  	ENDIF
      		  	IF(parm(2:2) == 't' .OR. parm(2:2) == 'T') imgtype = 'L'
      		  ELSEIF(J == 3) THEN
      		  	READ(parm, fmt='(I6)') ppi
      		  ELSEIF(J == 4) THEN
      		  	READ(parm, fmt='(I8)') header      		 
      		  ELSEIF(J == 5) THEN
      		  	READ(parm, fmt='(I8)') totwidth 
       		  ELSEIF(J == 6) THEN
      		  	READ(parm, fmt='(I8)') totheight 
      		  ELSEIF(J == 7) THEN
      		  	imgID = TRIM(parm) 
      		  ENDIF	
             END DO
   ELSE
       	     PAUSE 'Quitting:  Invalid Number of Arguments (must be 0, 1, 2, 3, or 6)'
       	     STOP
   END IF
  
   IF(runtype == 's') THEN
			k = LEN_TRIM(imgID)
			IF(imgID((k-3):k) == '.pgm' .OR. imgID((k-3):k) == '.PGM') THEN
				CALL PGM()	! read-in hdr,width,height of pgm format image
	  			IF(probflag == 1) THEN
	  				PAUSE 'Quitting:  cannot read image file'
	  				STOP
	  			ENDIF
	  		ENDIF
    ELSEIF(runtype=='f' .OR. runtype=='F') THEN
		imgtype = 'N'	! cardscan is always inked image
    ENDIF
  
 END SUBROUTINE COMMANDLINE
 
 
 
 

 

 

 

 
 
 