	MODULE	GlobalData	
	
!   ****************************    NOTICE   **************************** 
 
!    This "Image Quality of Fingerprint" (IQF) software was developed by 
!                            The MITRE Corporation
!    and was produced for the U.S. Government under Contract numbers 
!    J-FBI-12-128, W15P7T-05-C-F600 and W15P7T-07-C-F600 and is subject 
!    to the Rights in Data-General clause FAR 52.227-l4, Alt. IV (DEC 2007).
	
!   Redistribution and use in source and binary forms, with or without
!   modification, are permitted provided that the following conditions are met:
!    -Redistribution of source code must retain the above copyright notice,
!    this list of conditions, and the following disclaimer.
!    -Redistribution in binary form must reproduce the above copyright
!    notice, this list of conditions and the following disclaimer in the
!    documentation and/or other materials provided with the distribution.
	
!    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
!    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
!    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
!    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
!    OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
!    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
!    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
!    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
!    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
!    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
!    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
      
!   ********************************************************************* 
 
 
 
! This Module:  DEFINES PROGRAM-WIDE CONSTANTS AND ARRAYS
!		all trig function kernals in this program are in radians

       SAVE			

	  ! This cmdline vs dbl click is now performed in IQF.f95 based upon whether CMDLINE has be #defined
	  ! This parameter is no longer needed but is maintained in case others still want to use it.
       CHARACTER(1), PARAMETER :: cmdline = 'N'	
 				  !   'N'    compile as "dbl click" app with menu display in Windows or Mac OSX GUI mode
				  !    comment-out CALL COMMANDLINE() in IQF subroutine
				  !    delete COMMANDLINE.f95 from compiler files list
				  
				  !   'Y'    compile as true command line run in Mac or Windows/Cygwin terminal mode
				  !    activate CALL COMMANDLINE() in IQF subroutine
				  !    add COMMANDLINE.f95 to compiler files list
	
	CHARACTER(1), PARAMETER :: pgmwrite = 'Y'	! recreate entire input image with peak IQ window location		
		
	REAL, ALLOCATABLE, DIMENSION(:,:) :: power	! 2-D frequency domain real array 
	REAL, ALLOCATABLE, DIMENSION(:) :: DATAR	! RealImage & REAL part of FFT
	REAL, ALLOCATABLE, DIMENSION(:) :: DATAI	! Imaginary part of FFT
	REAL, ALLOCATABLE, DIMENSION(:) :: WORKR	! real work array within FFT
	REAL, ALLOCATABLE, DIMENSION(:) :: WORKI	! imaginary work array within FFT
	
	INTEGER(2), ALLOCATABLE, DIMENSION(:,:) :: image  ! two-dimensional array to hold entire image(col,row) 
 	INTEGER(2), ALLOCATABLE, DIMENSION(:,:) :: cardimage  ! two-dimensional array to hold entire cardscan
 	
	INTEGER, ALLOCATABLE, DIMENSION(:) :: xcoord, ycoord	! arrays holding UL corner location of all subimages in given image
	
	INTEGER, PARAMETER :: UIMG = 1			! input imagefile (read)
	INTEGER, PARAMETER :: UOUT = 4			! standard output file-concise or verbose (write)
	INTEGER, PARAMETER :: UAUX = 7	    		! input auxdatafile (read)
	INTEGER, PARAMETER :: UWRT = 10	    		! output subimage file (write)
	INTEGER, PARAMETER :: iqfnametype = 1		! 0=write over existing output file, 1=append to existing output file 
	INTEGER, PARAMETER :: maxbadimages = 100	! Program quits if number of unproccessable images exceeds this value
	INTEGER, PARAMETER :: maxheightpixels = 700	! maxfprint rectangle height referenced to 500ppi (700/500 =  1.4")
	INTEGER, PARAMETER :: MinWindowWidth = 200	! min window width (in pixels @ 500ppi)
	INTEGER, PARAMETER :: MaxWindowWidth = 400	! max window width 
							! To define a constant window width for all images, set:
							! MinWindowWidth = MaxWindowWidth 
	
	INTEGER, PARAMETER :: shiftpixels = 50		! square window (subimage) x,y shift increment referenced to 500ppi
							! 50/500 = 0.10" shift
	
	REAL, PARAMETER :: HVSparm = 2.*351.3288*8.726867E-03/.6  ! fixes HVS filter peak @ 0.5cy/pixelwidth (see computeIQ)
	
	REAL, PARAMETER :: widthfactor = .6	! width of square meas.window is 60% of found fingerprint width,
						! constrained by MinWindowWidth, MaxWindowWidth for inked or livescan image
	REAL, PARAMETER :: HCfactor = .17	! IQ adjustment factor for very High Contrast, binary image 
	REAL, PARAMETER :: maxIQ500 = 4850.	! max valid value for IQdc*avgGry**2  @ 500ppi                                                                                                              

	
     INTEGER :: ARGC             
	INTEGER :: totwidth, totheight, header  	! total image width & height (pixels) & header (bytes)
	INTEGER :: probflag, ppi			! ppi resolution level of fingerprint image (500, 1000, etc.) 		
	INTEGER :: xlength, ylength, Fprintwindow 	! all 3 parms: square subimage width in pixels 
	INTEGER :: ULrollcol(10), ULrollrow(10)		! upper left corners of the  10 rollblocks in ten-print cardscan
	INTEGER :: LRcol, LRrow				! lower right corner of rollblock #1 in ten-print cardscan
	
	CHARACTER (LEN=1) :: imgtype, runtype, order ! imgtype: latent or inked/live;   order:image polarity(L or B)

	CHARACTER (LEN=1), PARAMETER ::  SubImageInfo = 'N'   ! (Y)es or (N)o - printout info for Each subimage with run option d
	CHARACTER (LEN=10), PARAMETER :: iqfversion = 'v1.53'
	CHARACTER (LEN=20), PARAMETER :: iqfname = 'IQF_results'
	CHARACTER (LEN=100) ::  imgID, auxdataname, badfilelist(1000)   
	
	! establish smallest nonzero positive numbers for sngle & dble precision:
	DOUBLE PRECISION, PARAMETER :: TinyDble = 10.D0**(-RANGE(1.D0))
	REAL, PARAMETER :: TinySngl = 10.**(-RANGE(1.))
	
	
!	Window Widths to Avoid in 298 to 1198 pixel range (Slow Compute via Brenner FFT):
	INTEGER :: NGwindow(-4:186+5)	
	DATA NGwindow /1,1,1,1,1,298,302,314,326,332,334,346,356,358,362,366,382,384,386,388,394,398,402,404,410,412,422,424,&
	426,428,436,438,446,452,454,458,466,474,478,482,498,502,508,512,514,524,526,534,538,542,548,554,556,562,566,582,584,&
	586,596,604,606,614,618,622,626,628,632,634,640,642,652,654,662,664,668,674,678,692,694,698,706,712,716,718,724,734,&
	746,758,762,764,766,768,772,776,778,786,788,794,796,802,808,818,822,824,834,838,842,844,856,862,866,872,878,886,892,894,&
	896,898,904,906,908,914,916,922,926,932,934,942,956,958,960,964,970,974,978,982,998,1002,1004,1006,1010,1016,1018,1022,&
	1024,1028,1030,1038,1042,1046,1048,1052,1070,1074,1076,1082,1084,1086,1088,1090,1094,1096,1106,1108,1112,1114,1124,1126,&
	1130,1132,1136,1138,1142,1146,1152,1154,1158,1162,1164,1166,1168,1172,1174,1178,1180,1182,1184,1186,1192,1194,1198,&
	1,1,1,1,1/
	
	
	
	END MODULE GlobalData
