SUBROUTINE PGM ()

!   ****************************    NOTICE   **************************** 
 
!    This "Image Quality of Fingerprint" (IQF) software was developed by 
!                            The MITRE Corporation
!    and was produced for the U.S. Government under Contract numbers 
!    J-FBI-12-128, W15P7T-05-C-F600 and W15P7T-07-C-F600 and is subject 
!    to the Rights in Data-General clause FAR 52.227-l4, Alt. IV (DEC 2007).
	
!   Redistribution and use in source and binary forms, with or without
!   modification, are permitted provided that the following conditions are met:
!    -Redistribution of source code must retain the above copyright notice,
!    this list of conditions, and the following disclaimer.
!    -Redistribution in binary form must reproduce the above copyright
!    notice, this list of conditions and the following disclaimer in the
!    documentation and/or other materials provided with the distribution.
	
!    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
!    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
!    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
!    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
!    OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
!    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
!    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
!    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
!    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
!    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
!    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
      
!   ********************************************************************* 
 
 
 
 
                                                                                                                                  
! This subroutine: 
!  reads the image width & height in pixels, and header bytes,
!  for a pgm-formatted image ("P5" format, 8bpp grayscale image)
!  The image file can have MacOS9, MacOSX/Unix, or DOS/Windows linebreaks.

! General P5 pgm format:
!		P5
!		# a comment
!		width  height
!		maxgray
!		pixel values...

  
    USE globaldata
    
    	CHARACTER(len=1) :: LF, CR
	CHARACTER(len=2) :: linend
	CHARACTER(len=50) :: string
	CHARACTER(len=800) :: chunk		! assumes total header bytes < 800

	LOGICAL :: fileexists 
        
 				
 				! end-of-line:
 	LF = CHAR(10)		! Mac_OSX & Unix use LineFeed
	CR = CHAR(13)		! Mac_OS9 uses CarraigeReturn,  DOS/Windows uses CR followed by LF

    	    
    INQUIRE (file = imgID, exist = fileexists)         
    IF (.NOT. fileexists) GOTO 55        
!    PRINT*, 'Opening file ',imgID
    OPEN (unit=UIMG, file=imgID, action='READ', form='unformatted', recl=800, access='direct', err=59)
	!READ(unit=UIMG, rec=1, fmt='(A)', err=69) chunk
	READ(unit=UIMG,rec=1, err=69) chunk              !2014-09-18 F.Pertl: commented above read and removed 'fmt=(A)'
	IF(chunk(1:2) /= 'P5' .and. chunk(1:2) /= 'p5') GOTO 69

	IF(chunk(3:3) == LF) THEN
			linend = LF	
			ifirststart = 4
			itemp = 2
!			PRINT*, 'Mac OSX & Unix format'
	
	ELSEIF(chunk(3:3) == CR) THEN
			IF(chunk(4:4) == LF) THEN
					linend = CR//LF		
					ifirststart = 5
					itemp = 3
!					PRINT*, 'DOS/Windows format'
			ELSE
					linend = CR		
					ifirststart = 4
					itemp = 2
!					PRINT*, 'Mac OS9 format'
			ENDIF
	ELSE
			GOTO 89
	ENDIF
	
			iii=1
			istart = ifirststart
1			jjj = INDEX(chunk(istart:(istart+79)), TRIM(linend))  	! find next end-of-line after istart byte				
			iii=iii+1
			iend = (istart-1) + (jjj-1)
			IF(chunk(istart:istart) == '#') THEN	! line starting with # is just a comments line, skip it
				istart  = iend + itemp
				IF(iii > 40) THEN		!  if > 40 comment lines between P5 line & width,height line
					GOTO 93			!  then probably not true P5 pgm file
				ELSE 	
					GOTO 1
				ENDIF
			ELSE
				string = chunk(istart:iend)
				READ(string, *) totwidth, totheight
			ENDIF
			
			istart = iend + itemp		! this search assumes there are no more comment lines after width,height line
			jjj = INDEX(chunk(istart:(istart+79)), TRIM(linend)) 
			iend = (istart-1) + (jjj-1)
			header = iend + itemp - 1
!			PRINT*, 'header,width,height:  ', header, totwidth, totheight, imgID
			string = chunk(istart:iend)
			READ(string,*) maxgray
			IF(maxgray > 255) GOTO 95
!			ipixel= ICHAR(chunk(header+1:header+1))
!			PRINT*, 'ULcorner pixel gray=',ipixel

   	CLOSE(UIMG)
   	probflag = 2	! everything OK - filenname ends in  .pgm  and is readable
   	order = 'L'	! 'L' polarity is correct for most/all pgm images
   	RETURN		

   	

55  PRINT*, 'cannot find the pgm image file:  '
    GOTO 99

59  PRINT*, 'pgm image file open failed'
	GOTO 99

69  PRINT*, 'pgm image file header cannot be read, e.g., does not begin with P5'
	GOTO 99

89  PRINT*, 'pgm image  file linebreak type not recognized (is not Unix, Mac, or Windows type)'
	GOTO 99

93  PRINT*, 'more than 40 comment lines encountered, assumed not to be true pgm file'
	GOTO 99
	
95  PRINT*, 'maxgray in header > 255, is not true 8bpp grayscale pgm image'
	GOTO 99
	
99  CLOSE(UIMG)
 	probflag = 1	
 	PRINT*, '**** Skipping this image (imgfilename ends in .pgm but cannot read it): ',imgID
 	RETURN

   	END Subroutine PGM
  
  
  
