      SUBROUTINE readimage (itotwidth, itotheight, iheader, ipixel)  
                                                                                                                                  
!   ****************************    NOTICE   **************************** 
 
!    This "Image Quality of Fingerprint" (IQF) software was developed by 
!                            The MITRE Corporation
!    and was produced for the U.S. Government under Contract numbers 
!    W15P7T-05-C-F600 and W15P7T-07-C-F600 and is subject to the Rights in
!    Data General clause, at FAR 52.227-l4.
	
!   Redistribution and use in source and binary forms, with or without
!   modification, are permitted provided that the following conditions are met:
	
!    -Redistribution of source code must retain the above copyright notice,
!    this list of conditions, and the following disclaimer.
	
!    -Redistribution in binary form must reproduce the above copyright
!    notice, this list of conditions and the following disclaimer in the
!    documentation and/or other materials provided with the distribution.
	
!    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
!    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
!    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
!    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
!    OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
!    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
!    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
!    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
!    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
!    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
!    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
      
!   ********************************************************************* 
 
 
 
 
                                                                                                                                  
!  This subroutine:                                                                                                          
! 	Reads in the Entire image & stores in int*2, 2-D array: Image(col,row) 
!	Origin of entire image is at upper left corner of entire image at 1,1     
!	A subimage, however, has a Relative reference to entire image,
!	so if UL corner of subimage coincides with UL corner of entire image
!	then the subimage location reference is 0,0		
                                                                                                                                                               
!	Back in IQF subroutine, each specified subimage of the int*2, 2-D image array is 
!	re-stored as 1-D real array for input to FFT
                                                                                 
                                                         
                                                                                                                                  
      USE GLOBALDATA
	 
      INTEGER(2) :: dumy, reverse    
	                                                                                            
      CHARACTER(kind=1,len=(itotwidth*itotheight+iheader)), ALLOCATABLE, DIMENSION(:) :: wholeimage 
		! local variables: itotwidth,itotheight,iheader substituted for globals: totwidth,totheight,header
   
      LOGICAL :: fileexists 
      
      reverse = INT(255, kind(dumy))    
      
      INQUIRE (file = imgID, exist = fileexists)         
      IF (.NOT. fileexists) THEN
      		probflag = 1
      		PRINT*, '**** cannot find image file: ', imgID
      		RETURN
      ENDIF        

	
		  
 ! 8bpp IMAGE READ, STORE AS int*2, 2-D ARRAY
 
		ALLOCATE (wholeimage(1),stat=ierr) 
		IF(ierr /= 0) THEN 
			PRINT*, '**** Memory allocation failed for image read-in ', imgID
			probflag = 1
			DEALLOCATE (wholeimage)
			RETURN
     		ENDIF 			

		OPEN (unit=UIMG,file=imgID,action='READ',form='unformatted',access='direct',&
		  	recl=(itotwidth * itotheight + iheader), iostat=ierr)	  
	  	
		IF(ierr /= 0) THEN
			CLOSE(UIMG)
			PRINT*, '**** Image file open error     ',imgID
			probflag = 1
			DEALLOCATE (wholeimage)
			RETURN
		ENDIF

		READ(unit=UIMG,rec=1,iostat=ierr) wholeimage(1)
		IF(ierr /= 0) THEN
			probflag = 1
			PRINT*, '**** Image read error    ',imgID
			CLOSE(UIMG)
			DEALLOCATE (wholeimage)
			RETURN
		ENDIF
	  				
		
! 		READ IMAGE into int*2 array (assumes order='L'):
	  	DO irow = 1, itotheight
		     DO icol = 1, itotwidth
			     j = iheader + (irow-1) * itotwidth + icol
			     ijk = ICHAR(wholeimage(1)(j:j))
			     image(icol,irow) = INT(ijk, kind(dumy))
		     END DO
	  	END DO
		DEALLOCATE (wholeimage)  


!		if pgm image, order is set to 'L' in pgm subroutine of IQF v1.0 
!		if not pgm image, true order is unknown so initially let order = 'L'
		kt = LEN_TRIM(imgID)
		IF(imgID((kt-3):kt) /= '.pgm' .AND. imgID((kt-3):kt) /= '.PGM') THEN
			order = 'L'
		ELSE
			IF(order == 'B') THEN	! just in case pgm order was set to B
				DO i=1,totheight	
				   DO j=1,totwidth
					image(j,i) = reverse - image(j,i)
				   END DO
				END DO	
			ENDIF	
		ENDIF
		
	

! If input image is not pgm format, it is read as raw (even if it is tiff, etc.) & the true image polarity must then be established.
! The following logic attempts to identify the polarity based on some common properties of fingerprint images.
! i.e., most fingerprint images have light gray/white background (no fprint) near leftedge, bottom area.
! However, THIS LOGIC IS NOT FOOLPROOF.  User can check the program-selected polarity by verifying that gray level of pixel in
! upperleft corner of image is correct ("pix1" value that is output when option 'd' is selected); if not correct, 
! user can convert to pgm format image or reverse polarity of raw image, before input to program. 
	IF(imgID((kt-3):kt) /= '.pgm' .AND. imgID((kt-3):kt) /= '.PGM') THEN
		icount=0
		numcols = 25*ppi/500
		numrows = 10*ppi/500
		DO k = 78,98,10		! look at column-wise avg gray values of 3 small areas near leftedge, bottom
			jj = (REAL(k)*.01) * totheight	! top row at 78%, 88%, 98% of image height
			n = 0
			DO icol = 1, numcols
				isum = SUM(INT(image(icol,(jj-numrows+1):jj)))
				IF((isum/numrows) > 128 ) n = n+1
			END DO
			IF(n > (numcols/2)) icount=icount+1
		END DO
		IF(icount < 2) THEN	! only 0 or 1 of the 3 areas has more than half of the columns with avg gray > 128
			order = 'B'	! so initial polarity (L) is assumed to be wrong, so reverse it 
			DO i=1,totheight	
				DO j=1,totwidth
					image(j,i) = reverse - image(j,i)
				END DO
			END DO		
		ENDIF
	ENDIF
	

! 	Store pixel gray value for ULcorner of entire image (menu option d output):	
	ipixel = INT(image(1,1))
		

! Run Check: header+width*height should normally exactly equal total bytes in file
!	nbytes = header + totwidth * totheight 	
!	INQUIRE(unit=UIMG, FLEN = ifilesize)	! FLEN (filelength) is Absoft F95 extension
!	IF(ifilesize > nbytes) THEN	
!	 	PRINT*, ' '
!	 	PRINT*, '*** Caution: ImageFile has',ifilesize-nbytes, ' more bytes than sum of Hdrbytes + Pixelbytes   ', imgID
!	 	PRINT*, ' '      
!	ELSEIF(ifilesize < nbytes) THEN
!		probflag = 1
!	ENDIF
	
	CLOSE(UIMG)

END SUBROUTINE readimage  















					
			                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   
		
		
