SUBROUTINE WriteImage (peakcol, peakrow, peakavgGray)

!   ****************************    NOTICE   **************************** 
 
!    This "Image Quality of Fingerprint" (IQF) software was developed by 
!                            The MITRE Corporation
!    and was produced for the U.S. Government under Contract numbers 
!    J-FBI-12-128, W15P7T-05-C-F600 and W15P7T-07-C-F600 and is subject 
!    to the Rights in Data-General clause FAR 52.227-l4, Alt. IV (DEC 2007).
	
!   Redistribution and use in source and binary forms, with or without
!   modification, are permitted provided that the following conditions are met:
!    -Redistribution of source code must retain the above copyright notice,
!    this list of conditions, and the following disclaimer.
!    -Redistribution in binary form must reproduce the above copyright
!    notice, this list of conditions and the following disclaimer in the
!    documentation and/or other materials provided with the distribution.
	
!    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
!    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
!    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
!    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
!    OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
!    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
!    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
!    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
!    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
!    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
!    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
      
!   ********************************************************************* 
 
 
 
 
! This Subroutine:
! Writes the entire input image to a file, in PGM format, with a black or white boundary line 
! around the single square subimage that corresponds to the peak IQ
! This subroutine is invoked if program run is in "diagnostic" output mode

! Expected to be already defined: totwidth, totheight
! Parms carried in: peakcol, peakrow, Fprintwindow (upper left corner col,row of subimage & subimage width), peakavgGray 
! image(col,row) is entire image, 2-D, int*2 array
! Testing on pgm, raw, tiff input images indicates there should NOT be any special  
! accounting for polarity (order) when writing-out as pgm format.


 USE GLOBALDATA	 	
	    
 INTEGER :: peakcol, peakrow, peakcol11, peakrow11 
 INTEGER(2) :: dumy2
 INTEGER(4) :: dumy4
 
 CHARACTER (LEN=1) :: LF
 CHARACTER(LEN=14) :: numstring
 CHARACTER (LEN=100) :: outname

 LOGICAL :: file_exists    ! 2014-10-02 F.Pertl needed for checking file existance

 LF = CHAR(10)		! MacOSX & Unix linebreak, which is used for output pgm file

 peakrow11 = peakrow + 1      ! peakrow,peakcol from IQF subroutine are referenced to 0,0 because they refer to
 peakcol11 = peakcol + 1      ! a subimage, but in this subroutine need to convert to 1,1 origin for compatibility 
			      ! with range of image(col,row), which starts at 1,1, not 0,0
					   
 outname = TRIM(imgID)//'_IQ.pgm'


!OPEN(unit=UWRT, file=outname, form='unformatted', access='transparent', action='write', status='new',iostat=ierr)
!IF(ierr /= 0) THEN
!	CLOSE(UWRT)
! 	PRINT*, 'output image already exists, not recreated:  ', outname
!	RETURN
!ENDIF

 INQUIRE(file=outname,exist=file_exists)
 IF(file_exists) THEN
	CLOSE(UWRT)
 	PRINT*, 'output image already exists, not recreated:  ', outname
	RETURN
 ENDIF
 OPEN(unit=UWRT, file=outname, form='unformatted', access='stream', action='write')   !2014-10-02 F.Pertl transparent->stream and get rif of iostat

	
 line = 0  	 ! if avg gray of subimage >= 50 then black line will be most visible, otherwise use white line
 IF(peakavgGray < 50.) line = 255

! add PGM header code:   
 WRITE(numstring, fmt='(I0,1x,I0)') totwidth, totheight	    ! width&height but subimage is always square
 WRITE(UWRT) 'P5'//LF
 !WRITE(UWRT) ADJUSTL(numstring)//LF
 WRITE(UWRT) TRIM(numstring)//LF                            ! 2014-10-02  Remove both leading and trailing spaces.
 WRITE(UWRT) '255'//LF



!       create 2 pixel wide black or white boundary line around the square subimage:
	j = Fprintwindow - 1
	DO irow = MAX(MIN(peakrow,totheight),1), MAX(MIN(peakrow+j,totheight),1)
		DO icol = MAX(MIN(peakcol11,totwidth),1), MAX(MIN(peakcol11+j,totwidth),1)
			IF(irow==peakrow11 .or. irow==peakrow11+1 .or. irow==peakrow11+j-1 .or. &
			   irow==peakrow11+j .or. icol==peakcol11 .or. icol==peakcol11+1 .or. &
			   icol==peakcol11+j-1 .or. icol==peakcol11+j) THEN
					image(icol,irow) = INT(line, kind(dumy2))
			ENDIF
		END DO
	END DO



!     Write entire image to file, with boundary line around peak IQ subimage:
	DO irow = 1, totheight
		DO icol = 1, totwidth
			intpix = INT(image(icol,irow), kind(dumy4))
			IF(intpix > 255) intpix = 255
			IF(intpix < 0) intpix = 0
			WRITE(UWRT) CHAR(intpix)
            END DO
	END DO


CLOSE(UWRT)

END SUBROUTINE WriteImage

