SUBROUTINE CARDSCAN(meas_box_width, meas_box_height, cornerbad)



!   ****************************    NOTICE   **************************** 
 
!    This "Image Quality of Fingerprint" (IQF) software was developed by 
!                            The MITRE Corporation
!    and was produced for the U.S. Government under Contract numbers 
!    J-FBI-12-128, W15P7T-05-C-F600 and W15P7T-07-C-F600 and is subject 
!    to the Rights in Data-General clause FAR 52.227-l4, Alt. IV (DEC 2007).
	
!   Redistribution and use in source and binary forms, with or without
!   modification, are permitted provided that the following conditions are met:	
!    -Redistribution of source code must retain the above copyright notice,
!    this list of conditions, and the following disclaimer.
!    -Redistribution in binary form must reproduce the above copyright
!    notice, this list of conditions and the following disclaimer in the
!    documentation and/or other materials provided with the distribution.
	
!    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
!    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
!    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
!    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
!    OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
!    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
!    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
!    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
!    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
!    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
!    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
      
!   ********************************************************************* 


! This Subroutine:
! Finds UpperLeft corner of each rollblock in a tenprint cardscan
! (after safety factor is applied), resulting in:  ULcol, ULrow. 
! Each rollblock ULcorner is referenced to lower right corner of rollblock #1, which is either 
! determined by the user before program run, or else automatically by subroutine "FindCorner".
! For non-standard tenprint cards with double horizontal frameline in each rollblock, FindCorner
! may be confused as to which frameline to use.

! Output meas_box_width = 1.6" x ppi x (1-safetyfactor)
! 	 meas_box_height = 1.5" x ppi x (1-safetyfactor)
! Tenprint card image can be either full cardscan (8"x8") or 14 printblock area (8"x5"),
! or 10 rollblock area (8"x3").


! Back in iqf subroutine, each rollblock location in cardscan is treated as a separate image for processing
! (imgtype=inked, Fprint subroutine applied).



USE GlobalData


INTEGER :: pblockcol(10), pblockrow(10), wide, high
INTEGER :: offsetcol, offsetrow, meas_box_width, meas_box_height
INTEGER :: flagcol, flagrow, startblok, endblok

CHARACTER (LEN=1) :: refmethod, cornerbad
CHARACTER (LEN=2) :: boxflag                   !2014-09-97 F.Pertl: added character length 2 declare statement





!  ********** CONSTANTS & INITIAL VALUES 
cornerbad = 'N'         ! initially assume all corners found are OK								
winch = 1.6		! standard rollblock width
hinch = 1.5		! standard rollblock height
rppi = REAL(ppi)
pblockcol = 0		! upperleft col of each rollblock
pblockrow = 0		! upperleft row of each rollblock
ULrollcol = 0		! upperleft col of measurement box in each rollblock
ULrollrow = 0		! upperleft row of measurment box in each rollblock
safety = .12		! 1.-safety = fraction of printblock width,height used (pblockcol=ULrollcol if safety=0.)


IF(LRcol == 0 .AND. LRrow == 0) THEN
	refmethod = 'A'		! LRcorner of pblock#1 to be located by program 
	offsetcol = 0
	offsetrow = 0

ELSE
	refmethod = 'M'		! LRcorner of pblock#1 was located by user (before program run)
	offsetcol = LRcol - NINT(rppi*winch)
	offsetrow = LRrow - NINT(rppi*hinch)

ENDIF




!   Define Upper Left Corner of Each Rollblock:
!   wide, high = rollblock width, height in pixels
!   pblockcol, pblockrow = col,row of UL corner of pblock (ref: 1,1 for pblock#1)
!   meas_box_width, meas_box_height = width, height of measurement box within rollblock, in pixels

	
	DO  i = 1, 10	! 10 rollblocks
		wide = INT(winch*rppi)		
		high = INT(hinch*rppi)
		IF(i < 6) THEN
			pblockrow(i) = 1
			pblockcol(i) = (i-1)*wide + 1
		ELSE
			pblockrow(i) = high + 1
			pblockcol(i) = pblockcol(i-5)
		ENDIF		
	END DO
	meas_box_width = INT((1. - safety)*REAL(wide))	
	meas_box_height = INT((1. - safety)*REAL(high))
	IF(meas_box_width > wide) meas_box_width = wide
	IF(meas_box_height > high) meas_box_height = high





!	FIND LOWER RIGHT CORNER OF ROLLBLOCK #1 (LRcol, LRrow)		
	IF(refmethod == 'A') then	
		CALL FINDCORNER(rppi, flagcol, flagrow)
		offsetcol = LRcol - NINT(rppi*winch)
		offsetrow = LRrow - NINT(rppi*hinch)
		
		IF(flagcol == 0 .or. flagrow == 0) then
!			PRINT*, 'UNSUCCESSFUL auto corner search, will use nominal corner col,row values: ',imgID
		ELSEIF(flagcol == 3 .or. flagrow == 3) then
!			PRINT*, 'Corner found is suspect - Verify: ',imgID
		ENDIF
	ENDIF

	
	
	! Define UpperLeft Corner of Measurement Box in Each Rollblock (ULrollcol, ULrollrow)
	! Given LowerRight Corner of Rollblock #1 at LRcol, LRrow (previously found by FindCorner or user input)
	! iset=1 for blocks 1-5;   iset=2 for blocks 6-10
	DO  iset = 1, 2        
			       
		IF(iset == 1) THEN
			startblok = 1	! top row of rollblocks, 1-5
			endblok = 5
		ELSE
			startblok = 6	! bottom row of rollblocks, 6-10
			endblok = 10
		ENDIF
									
		DO  iblok = startblok, endblok     
														
			IF(iblok == 1 .OR. iblok == 6) THEN
				indent = 0
			ELSE
				indent = indent + wide
			ENDIF
			
			! Upper Left Corner of measurement box defined by:    ULrollcol, ULrollrow
			ULrollcol(iblok) = INT((wide - meas_box_width)/2) + offsetcol + indent +1
			ULrollrow(iblok) = (iset-1)*high + offsetrow + INT((high - meas_box_height)/2) +1
			
			! test to see if meas.box is totally within image, 
			! i.e. is not < 0 col,row  or > width col or  > height row
			! if any value is outside image, redefine ULrollcol or ULrollrow to bring within image area
			boxflag = '  '
            IF(ULrollcol(iblok) < 1) THEN
				ULrollcol(iblok) = 1
				boxflag = '**'
			ELSEIF(ULrollrow(iblok) < 1) THEN
				ULrollrow(iblok) = 1
				boxflag = '**'
			ELSEIF((ULrollcol(iblok) + meas_box_width - 1) > totwidth) THEN
				ULrollcol(iblok) = totwidth - meas_box_width
				boxflag = '**'
			ELSEIF((ULrollrow(iblok) + meas_box_height - 1) > totheight) THEN
				ULrollrow(iblok) = totheight - meas_box_height
				boxflag = '**'
			ENDIF
			IF(boxflag == '**') cornerbad = 'Y'
		END DO		
	END DO		
		

RETURN
END SUBROUTINE CARDSCAN







! *****************************************************************************

SUBROUTINE FINDCORNER(rppi, flagcol, flagrow)

! FINDS LRcol, LRrow:  COL,ROW OF LOWER RIGHT CORNER OF ROLLBLOCK #1
! BY SEARCHING FOR inverted 'L' PATTERN FORMED BY COMMON BOUNDARY INTERSECTION OF BLOCKS 2,6,7
! (vertical leg is blocks 6&7 boundaryline, horizontal leg is blocks 2&7 boundaryline)
! Corner locater uses avg gray & gray variance of all selected pixels in given col,row combination.
! Reasoning: true boundary line should have lowest avg gray & lowest variance of all
! col,row locations tested for inverted 'L' pattern.
! 		First search is for minimum average gray for inverted L candidate
!		Second search is for minimum gray variance for inverted L candidate
!		Answer is chosen from first or second search, depending on which col,row 
!		is closer to nominal corner location: col=1.6", row=1.5", 
!		therefore, choice ASSUMES correct answer should be "close" to col=1.6", row=1.5" 
!		(note: tried combining for min avg & min var as third test, 
!		       but is no more accurate than separate avg & var tests)
! image(col,row) array is 8 bpp image pixel gray levels (integer*2)
! Image must be in normal tenprint card viewing orientation; skewangle < apprx. 1.5 degrees.
! Image can be either entire card or just 14 printblock or just 10 rollblocks area.
! rppi = nominal scanner ppi, normally = 500. or 1000. (exact value not needed)

USE GlobalData


INTEGER :: LRcol1, LRcol2, LRrow1, LRrow2 
INTEGER :: colstart, rowstart, row, col
INTEGER :: textblock, flagcol, flagrow


!       INITIAL VALUES:
searchsize = 1.3         	! #inches in vert(row) & horz(col) directons to search for corner;
				! 1.3 inch implies search starts 1.3/2 = .65" above and .65"to the left of nominal
				! corner @ col=800, row=750 pixels (for 500ppi)
ipixsize = INT(searchsize * rppi)    ! x,y search size in pixels, for 'searchsize' in inches
numpix = NINT(.5*rppi)       	 ! if a row & intersecting column, each numpix long, are each "blacker"
				! then surround, then a candidate inverted 'L' location has been found; 
				! the length chosen for numpix can be critical to success; optimum value
				! is dependent on type of image being assessed;
				! .5*ppi = 1/2 inch leg length, good for 10-print cards

IF(totwidth*totheight > (ppi*ppi*7*7)) THEN
  	textblock = 3 * ppi             ! 8x8 inch image - need to skip top 2.8 inches
ELSE
  	textblock = 0                      		! 8x5 inch or 8x3 inch image - no skip
ENDIF
flagcol = 0  	! if flag still = 0 at end of this subroutine, corner was not found
flagrow = 0	! flag = 1 at end, corner found using search for minimum average
		! flag = 2 at end, corner found using search for minimum variance
		! flag = 3 at end, corner found but may be wrong (is >0.5" from 1.6"/1.5" corner)
colstart = NINT(1.6*rppi) - ipixsize/2					! starting column for search
rowstart = textblock + NINT(1.5*rppi) - ipixsize/2 		! starting row for search

LRcol = NINT(1.6*rppi)					! nominal corner col value in case search fails
LRrow = textblock + NINT(1.5*rppi)		! nominal corner row value in case search fails
inc = 1		! col & row increments for inverted "L" search; critical value,tied to nominal line width
			! being searched for; for fprint card scans @ 500ppi,pblock line is only 4-7 pixels wide;
			! prior to v2.1, used inc=2 which Usually was adequate; if inc > 2 may miss line.
incsum = 1	! for given col & row, every pixel in col,row is used to compute avg & variance
			! incsum could = 2 or 4 (faster compute) but not quite as accurate
samples = 2.*REAL(numpix/incsum +1)	! off by 1 when numpix/incsum has no remainder


! Search to find minimum avgray and (separately) min variance for 
! VERTICAL leg of candidate inverted L pattern:
	previousavg = 1000.
	previousvar = 1.E10
	DO  col = colstart, colstart+ipixsize, inc        ! search cols for vertline of corner
		DO  row = rowstart, rowstart+ipixsize, inc    ! search rows for horzline of corner
			icol = col
			irow = row
			CALL STATS(numpix,incsum,'V',icol,irow,avg,var)
			IF(avg < 180.) then			! true black line should be < 180 gray
				IF(avg < previousavg ) then
	  				LRcol1 = col
					previousavg = avg
	  			ENDIF
				IF(var < previousvar) then
	  				LRcol2 = col
					previousvar = var
	  			ENDIF
			ENDIF
		END DO
	END DO
! Determine which of above 2 col search results are closer to nominal col=800 @500ppi or col=1600 @1000ppi:
IF(ABS(LRcol1 - 1.6*rppi) > abs(LRcol2 - 1.6*rppi)) THEN
	LRcol = LRcol2
	flagcol = 2
ELSE
	LRcol = LRcol1
	flagcol = 1
ENDIF



! Search to find minimum avgray and (separately) min variance for  
! HORIZONTAL leg of candidate inverted L pattern:
	col = LRcol			! search is down previously found column only
	previousavg = 1000.
	previousvar = 1.E10
		DO  row = rowstart, rowstart+ipixsize, inc    ! search rows for horzline of corner
			icol = col
			irow = row
			CALL STATS(numpix,incsum,'H',icol,irow,avg,var)
			IF(avg < 180.) then			! true black line should be < 180 gray
				IF(avg < previousavg ) then
					LRrow1 = row
					previousavg = avg
	  			ENDIF
				IF(var < previousvar) then
					LRrow2 = row
					previousvar = var
	  			ENDIF
			ENDIF
		END DO
	
! Determine which of above 2 row search results are closer to nominal row=750 corner:
IF(ABS(LRrow1 - 1.5*rppi) > ABS(LRrow2 - 1.5*rppi)) THEN
	LRrow = LRrow2
	flagrow = 2
ELSE
	LRrow = LRrow1
	flagrow = 1
ENDIF

IF(flagcol /= 0 .and. flagrow /= 0) THEN
	! if found corner is > 1/2" away from nominal value (1.6",1.5"), then ans.may be wrong: 
	IF(ABS(1.6-REAL(LRcol)/rppi) > .5)  flagcol = 3
	IF(ABS(1.5-REAL(LRrow-textblock)/rppi) > .5 ) flagrow = 3
ENDIF

RETURN
END SUBROUTINE FINDCORNER



! *****************************************************************************************



SUBROUTINE STATS(numpix,incsum,direction,icol,irow,avg,var)

! Find gray average & variance, for one leg of inverted 'L' 
! direction=V for vertical leg,     direction=H for horizontal leg
! numpix, incsum, direction, icol, irow    are values carried into STATS
! avg, var    are values computed by Stats & carried back to FINDCORNER


USE GlobalData


INTEGER :: sumcolrow, sum2


CHARACTER(1) :: direction
		
	sumcolrow = 0
	sum2 = 0
	n = 0;  ic = 0;  ir = 0
	DO  i = 1, numpix, incsum	
  		IF(direction == 'V') THEN		! avg & var of column
			ic = image(icol, irow+i)
		ELSE						! avg & var of row
			ir = image(icol+i, irow)
		ENDIF
		sumcolrow = sumcolrow + ic + ir
		sum2 = sum2 + ic*ic + ir*ir
		n = n+1
	END DO
	avg = REAL(sumcolrow)/REAL(n)
	var = (REAL(n) * REAL(sum2) - REAL(sumcolrow)**2) / REAL(n*(n-1))		

RETURN
END SUBROUTINE STATS

